// Win32Configurator.cc --- Configuration Access
//
// Based on Win32Configurator.cc from WorkRave
// Copyright (C) 2002 Raymond Penners <raymond@dotsphinx.com>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// Modified for Gabber by Thomas Muldowney
//

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "debug.hh"
#include "Win32Configurator.hh"

using namespace Gabber;

Win32Configurator::Win32Configurator() : key_root("Software/gabber")
{
}


Win32Configurator::~Win32Configurator()
{
}


bool Win32Configurator::load(const Glib::ustring& filename)
{
  return true;
}


bool Win32Configurator::save(const Glib::ustring& filename)
{
  return true;
}


bool Win32Configurator::save()
{
  return true;
}


bool Win32Configurator::get_value(const Glib::ustring& key, Glib::ustring& out) const
{
  HKEY handle;
  bool rc = false;
  Glib::ustring k, p, p32, c;
  DWORD disp;
  LONG err;

  k = key_add_part(key_root, key);
  key_split(k, p, c);
  p32 = key_win32ify(p);
  err = RegOpenKeyEx(HKEY_CURRENT_USER, p32.c_str(), 0, KEY_ALL_ACCESS, &handle);
  if (err == ERROR_SUCCESS)
    {
      DWORD type, size;
      char buf[256]; // FIXME: yuck, should be dynamic.
      size = sizeof(buf);
      err = RegQueryValueEx(handle, c.c_str(), 0, &type, (LPBYTE) buf, &size);
      if (err == ERROR_SUCCESS)
        {
          out = buf;
          rc = true;
        }
      RegCloseKey(handle);
    }
  return rc;
}


bool Win32Configurator::get_value(const Glib::ustring& key, bool& out) const
{
  long l;
  bool rc = get_value(key, &l);
  if (rc)
    {
      out = (bool) l;
    }
  return rc;
}


//! Returns the value of the specified attribute
/*!
 *  \retval true value successfully returned.
 *  \retval false attribute not found.
 */
bool Win32Configurator::get_value(const Glib::ustring& key, int& out) const
{
  long l;
  bool rc = get_value(key, &l);
  if (rc)
    {
      out = (int) l;
    }
  return rc;
}


//! Returns the value of the specified attribute
/*!
 *  \retval true value successfully returned.
 *  \retval false attribute not found.
 */
bool Win32Configurator::get_value(const Glib::ustring& key, long& out) const
{
  Glib::ustring s;
  bool rc = get_value(key, &s);
  if (rc)
    {
      int f = sscanf(s.c_str(), "%ld", out);
      rc = (f == 1);
    }
  return rc;
}


//! Returns the value of the specified attribute
/*!
 *  \retval true value successfully returned.
 *  \retval false attribute not found.
 */
bool Win32Configurator::get_value(Glib::ustring& key, double& out) const
{
  Glib::ustring s;
  bool rc = get_value(key, &s);
  if (rc)
    {
      int f = sscanf(s.c_str(), "%f", out);
      rc = (f == 1);
    }
  return rc;
}


bool Win32Configurator::set_value(Glib::ustring& key, Glib::ustring v)
{
  TRACE_ENTER_MSG("Win32Configurator::set_value", key << "," << v);

  HKEY handle;
  bool rc = false;
  Glib::ustring k, p, p32, c;
  DWORD disp;
  LONG err;

  k = key_add_part(key_root, key);
  key_split(k, p, c);
  p32 = key_win32ify(p);
  err = RegCreateKeyEx(HKEY_CURRENT_USER, p32.c_str(), 0,
                       "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
                       NULL, &handle, &disp);
  if (err == ERROR_SUCCESS)
    {
      err = RegSetValueEx(handle, c.c_str(), 0, REG_SZ, (BYTE *) v.c_str(), v.length()+1);
      RegCloseKey(handle);
      rc = (err == ERROR_SUCCESS);
      if (rc)
	{
	  fire_configurator_event(key);
	}
    }
  return rc;
}




bool Win32Configurator::set_value(Glib::ustring& key, int v)
{
  char buf[32];
  sprintf(buf, "%d", v);
  return set_value(key, Glib::ustring(buf));
}

bool Win32Configurator::set_value(Glib::ustring& key, long v)
{
  char buf[32];
  sprintf(buf, "%ld", v);
  return set_value(key, Glib::ustring(buf));
}

bool Win32Configurator::set_value(Glib::ustring& key, bool v)
{
  char buf[32];
  sprintf(buf, "%d", v ? 1 : 0);
  return set_value(key, Glib::ustring(buf));
}


bool Win32Configurator::set_value(Glib::ustring& key, double v)
{
  char buf[32];
  sprintf(buf, "%f", v);
  return set_value(key, Glib::ustring(buf));
}


bool Win32Configurator::exists_dir(Glib::ustring& key) const
{
  TRACE_ENTER_MSG("Win32Configurator::exists_dir", key);
  HKEY handle;
  bool rc = false;
  string k, k32;
  DWORD disp;
  LONG err;

  k = key_add_part(key_root, key);
  k32 = key_win32ify(k);
  err = RegOpenKeyEx(HKEY_CURRENT_USER, k32.c_str(), 0, KEY_READ, &handle);
  if (err == ERROR_SUCCESS)
    {
      rc = true;
      RegCloseKey(handle);
    }
  return rc;
}






list<Glib::ustring> Win32Configurator::get_all_dirs(Glib::ustring& key) const
{
  TRACE_ENTER_MSG("Win32Configurator::get_all_dirs", key);
  list<Glib::ustring> ret;
  LONG err;
  HKEY handle;

  Glib::ustring k = key_add_part(key_root, key);
  Glib::ustring k32 = key_win32ify(k);
  err = RegOpenKeyEx(HKEY_CURRENT_USER, k32.c_str(), 0,
		     KEY_ALL_ACCESS, &handle);
  TRACE_MSG(k32);
  if (err == ERROR_SUCCESS)
    {
      for (int i = 0; ; i++)
	{
	  // FIXME: static length, eek
	  char buf[256];
	  DWORD buf_size = sizeof(buf);
	  FILETIME time;
      
	  err = RegEnumKeyEx(handle, i, buf, &buf_size, NULL, NULL, NULL, &time);
	  if (err == ERROR_SUCCESS)
	    {
	      string s = buf;
	      ret.push_back(s);
	    }
	  else
	    {
	      break;
	    }
	}
      RegCloseKey(handle);
    }
  return ret;  
}


Glib::ustring Win32Configurator::key_add_part(const Glib::ustring& s, 
                                              const Glib::ustringY t) const
{
  Glib::ustring ret = s;
  add_trailing_slash(ret);
  return ret + t;
}

void Win32Configurator::key_split(const Glib::ustring key, 
                                  Glib::ustring &parent, 
                                  Glib::ustring &child) const
{
  const char *s = key.c_str();
  char *slash = strrchr(s, '/');
  if (slash)
    {
      parent = key.substr(0, slash-s);
      child = slash+1;
    }
  else
    {
      parent = "";
      child = "";
    }
}

Glib::ustring Win32Configurator::key_win32ify(Glib::ustring& key) const
{
  Glib::ustring rc = key;
  strip_trailing_slash(rc);
  for (int i = 0; i < rc.length(); i++)
    {
      if (rc[i] == '/')
        {
          rc[i] = '\\';
        }
    }
  return rc;
}


